@@ -19,23 +19,56 @@ import logging
 logger = logging.getLogger(__name__)
 
 
+def np_arr_img_to_bytes(arr, format='JPEG') -> bytes:
+    return image_to_bytes(Image.fromarray(arr, 'RGB'), format=format)
+
+
+def image_to_bytes(image: Image, format="JPEG") -> bytes:
+    with BytesIO() as bytes:
+        image.save(bytes, format=format)
+        return bytes.getvalue()
+
+
 class Images(Entities):
+    """
+    **The object mapper representation of images in ApertureDB.**
+
+    This class is a layer on top of the native query.
+    It facilitate interactions with images in the database in pythonic way.
+
+    It abstracts the need to batch the responses, and includes utility methods to
+    interconvert the representation into numpy matrices, and find similar images,
+    related bounding boxes, etc.
+    """
     db_object = "_Image"
 
     def inspect(self, use_thumbnails=True) -> Union[Tuple[widgets.IntSlider, widgets.Output], DataFrame]:
         df = super(Images, self).inspect()
         if use_thumbnails:
+            def image_base64(im):
+                with BytesIO() as buffer:
+                    im.save(buffer, 'jpeg')
+                    return base64.b64encode(buffer.getvalue()).decode()
+
+            def get_formatters(width):
+                def image_formatter(im):
+                    return f'<img width={width} style="max-width: 400px" src="data:image/jpeg;base64,{image_base64(im)}" >'
+
+                return image_formatter
+
             sizer = widgets.IntSlider(min=1, max=400, value=100)
             op = widgets.Output()
 
-            def widget_interaction(c):
-                def image_base64(im):
-                    with BytesIO() as buffer:
-                        im.save(buffer, 'jpeg')
-                        return base64.b64encode(buffer.getvalue()).decode()
+            with op:
+                image_formatter = get_formatters(sizer.value)
+                display(HTML("<div style='max-width: 100%; overflow: auto;'>" +
+                             df.to_html(
+                                 formatters={'thumbnail': image_formatter}, escape=False)
+                             + "</div>")
+                        )
 
-                def image_formatter(im):
-                    return f'<img width={c["new"]} style="max-width: 400px" src="data:image/jpeg;base64,{image_base64(im)}" >'
+            def widget_interaction(c):
+                image_formatter = get_formatters(c['new'])
                 with op:
                     op.clear_output()
                     display(HTML("<div style='max-width: 100%; overflow: auto;'>" +
@@ -78,8 +111,6 @@ class Images(Entities):
         self.format      = None
         self.limit       = None
 
-        self.search_result = None
-
         self.adjacent = {}
 
         self.batch_size = batch_size
@@ -90,6 +121,10 @@ class Images(Entities):
         self.bbox_label_prop = "_label"
         self.get_image = True
 
+        if response is not None:
+            self.images_ids = list(
+                map(lambda x: x[self.img_id_prop], response))
+
     def __retrieve_batch(self, index):
         '''
         **Retrieve the batch that contains the image with the specified index**
@@ -252,11 +287,23 @@ class Images(Entities):
             print(self.db_connector.get_last_response_str())
             raise
 
-    def total_results(self):
+    def total_results(self) -> int:
+        """
+        **Returns the total number of images that matched the query**
+
+        Returns:
+            int: Count of images that match the query.
+        """
 
         return len(self.images_ids)
 
-    def get_image_by_index(self, index):
+    def get_image_by_index(self, index: int):
+        """**Get a single image by it's index in the array of retrieved ids**
+
+        Args:
+            index (int): Position in the image ids retrieved.
+
+        """
 
         if index >= len(self.images_ids):
             logger.info("Index is incorrect")
@@ -273,7 +320,13 @@ class Images(Entities):
 
         return self.images[str(uniqueid)]
 
-    def get_np_image_by_index(self, index):
+    def get_np_image_by_index(self, index: int):
+        """**Retrieves the numpy representation of image from database**
+
+        Args:
+            index (int): Position in the image ids retrieved.
+
+        """
 
         image = self.get_image_by_index(index)
         # Just decode the image from buffer
@@ -284,7 +337,16 @@ class Images(Entities):
 
         return image
 
-    def get_bboxes_by_index(self, index):
+    def get_bboxes_by_index(self, index: int):
+        """
+        **Get related bounding box for the image**
+
+        Args:
+            index (int): Position in the image ids retrieved.
+
+        Returns:
+            _type_: _description_
+        """
         if not self.images_bboxes or not str(self.images_ids[index]) in self.images_bboxes:
             # Fetch when not present in the map.
             self.__retrieve_bounding_boxes(index)
@@ -299,6 +361,16 @@ class Images(Entities):
 
     # A new search will throw away the results of any previous search
     def search(self, constraints=None, operations=None, format=None, limit=None, sort=None):
+        """
+        **Sets a new query to retrieve images parameters as described below**
+
+        Args:
+            constraints (dict, optional): [Constraints](/python_sdk/parameter_wrappers/Constraints). Defaults to None.
+            operations (dict, optional): [Operations](/python_sdk/parameter_wrappers/Operations). Defaults to None.
+            format (str, optional): Format of the returned images. Without this, the images are returned as they are stored. Defaults to None.
+            limit (int, optional): number of values to restrict results to. Defaults to None.
+            sort (dict, optional): [Sort](/python_sdk/parameter_wrappers/Sort). Defaults to None.
+        """
 
         self.constraints = constraints
         self.operations  = operations
@@ -345,9 +417,16 @@ class Images(Entities):
         except:
             print("Error with search: {}".format(response))
 
-        self.search_result = response
+        self.response = entities
 
-    def search_by_property(self, prop_key, prop_values):
+    def search_by_property(self, prop_key: str, prop_values: list):
+        """
+        **Constructs a constraints block and does a new search**
+
+        Args:
+            prop_key (str): Key on which to search
+            prop_values (list): The values that must match for the key.
+        """
         const = Constraints()
         const.is_in(prop_key, prop_values)
         img_sort = {
@@ -418,7 +497,6 @@ class Images(Entities):
                     "k_neighbors": n_neighbors + 1,
                     "blobs":     False,
                     "distances": True,
-                    "ids":       True,
                     "uniqueids": True,
                 }
             }, {
@@ -618,7 +696,7 @@ class Images(Entities):
             image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
 
             fig1, ax1 = plt.subplots()
-            plt.imshow(image), plt.axis("off")
+            plt.imshow(image)
 
     def get_props_names(self):
 
