@@ -1,3 +1,7 @@
+"""
+**These are miscelaneous helper methods to render responses from ApertureDB in a jupyter environment.**
+"""
+
 import os
 from typing import List
 import cv2
@@ -6,8 +10,8 @@ import numpy as np
 from PIL import Image
 from IPython.display import Video
 from IPython.display import display as ds
-from IPython.display import HTML
 from base64 import b64encode
+import matplotlib.pyplot as plt
 
 
 DESTINATION_FOLDER = "results"
@@ -23,12 +27,20 @@ class Rectangle:
 
 
 class BoundingBox:
+    """
+    **A class which combines a box and a label.**
+    """
+
     def __init__(self, r: Rectangle = None, label: str = None) -> None:
         self.r = r
         self.label = label
 
 
 class TemporalBoundingBox:
+    """
+    **A class that represents a box, but also associated time in the range of frame start and end**
+    """
+
     def __init__(self, bb: BoundingBox = None, start_frame: int = 0, end_frame = 0) -> None:
         self.bb = bb
         self.start_frame = start_frame
@@ -41,12 +53,29 @@ def check_folder(folder):
 
 
 def display(images_array, save=False):
+    """
+    Show images with matplotlib
+    :::note
+    This method was used by ApertureDB to display images, but it is not recommended anymore.
+    It will not draw annotation.
+
+    Instead, when using with JSON queries, make a instance of the Images class and call the display method.
+    :
+
+    ```python
+    from aperturedb.Images import Images
+    result, response, blobs = execute_batch([{"FindImage":{"uniqueids": True}}], [], db)
+    wrapper = Images(db, response=response[0]["FindImage"]["entities"])
+    wrapper.display()
+    ```
+    """
 
     for im in images_array:
         nparr = np.fromstring(im, np.uint8)
         image = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
         image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
-        ds(Image.fromarray(image))
+        plt.subplots()
+        plt.imshow(image)
 
     if save:
         check_folder(DESTINATION_FOLDER)
@@ -93,12 +122,25 @@ def save_video(blob, name):
 
 
 def display_video_mp4(blob):
+    """
+    **Display a video using IPython.display**
+
+    Args:
+        blob (bytearray): A blob that is the Video.
+    """
     name = DESTINATION_FOLDER + "/" + "video_tmp.mp4"
     save_video(blob, name)
     ds(Video(name, embed=True))
 
 
-def annotate_video(blob, boxes=[], tags=[], bboxes: List[TemporalBoundingBox] = []):
+def annotate_video(blob, bboxes: List[TemporalBoundingBox] = []):
+    """
+    **Place annotations on a video framewise**
+
+    Args:
+        blob (bytearray): Video blob returned from the database
+        bboxes (List[TemporalBoundingBox], optional): List of boxes to be drawn. Defaults to [].
+    """
     name = DESTINATION_FOLDER + "/" + "video_tmp.mp4"
     save_video(blob, name)
     cap = cv2.VideoCapture(name)
@@ -143,12 +185,12 @@ def annotate_video(blob, boxes=[], tags=[], bboxes: List[TemporalBoundingBox] =
         print("Unable to open cap")
 
 
-def display_annotated_video(blob, boxes=[], tags=[], bboxes: List[TemporalBoundingBox] = []):
+def display_annotated_video(blob, bboxes: List[TemporalBoundingBox] = []):
     """
     Returns a HTML representation with a column filled
     with video entities.
     """
-    annotate_video(blob, boxes, tags, bboxes)
+    annotate_video(blob, bboxes)
     video_path = os.path.join(DESTINATION_FOLDER, 'output.mp4')
     mp4 = open(video_path, "rb").read()
     data_url = "data:video/mp4;base64," + b64encode(mp4).decode()
