@@ -13,8 +13,9 @@ from aperturedb.DaskManager import DaskManager
 logger = logging.getLogger(__name__)
 
 
-def execute_batch(q, blobs, db,
-                  response_handler: Callable = None, commands_per_query: int = 1, blobs_per_query: int = 0):
+def execute_batch(q, blobs, db, success_statuses: list[int] = [0],
+                  response_handler: Callable = None, commands_per_query: int = 1, blobs_per_query: int = 0,
+                  strict_response_validation: bool = False):
     """
     Execute a batch of queries, doing useful logging around it.
     Calls the response handler if provided.
@@ -45,7 +46,7 @@ def execute_batch(q, blobs, db,
                 b_count = 0
                 for req, resp in zip(q[start:end], r[start:end]):
                     for k in req:
-                        # Ref to https://docs.aperturedata.io/parameters/blobs.html
+                        # Ref to https://docs.aperturedata.io/query_language/Reference/shared_command_parameters/blobs
                         blobs_where_default_true = \
                             k in ["FindImage", "FindBlob"] and (
                                 "blobs" not in req[k] or req[k]["blobs"])
@@ -64,18 +65,37 @@ def execute_batch(q, blobs, db,
                         b[blobs_returned:blobs_returned + b_count] if len(b) < blobs_returned + b_count else None)
                 except BaseException as e:
                     logger.exception(e)
+                    if strict_response_validation:
+                        raise e
                 blobs_returned += b_count
     else:
         # Transaction failed entirely.
         logger.error(f"Failed query = {q} with response = {r}")
         result = 1
-    if isinstance(r, dict) and db.last_response['status'] != 0:
-        logger.error(f"Failed query = {q} with response = {r}")
+
+    statuses = {}
+    if isinstance(r, dict):
+        statuses[r['status']] = [r]
+    elif isinstance(r, list):
+        # add each result to a list of the responses, keyed by the response
+        # code.
+        [statuses.setdefault(result[cmd]['status'], []).append(result)
+         for result in r for cmd in result]
+    else:
+        logger.error("Response in unexpected format")
         result = 1
-    if isinstance(r, list) and not all([v['status'] == 0 for i in r for k, v in i.items()]):
-        logger.warning(
-            f"Partial errors:\r\n{json.dumps(q)}\r\n{json.dumps(r)}")
-        result = 2
+
+    # last_query_ok means result status >= 0
+    if result != 1:
+        warn_list = []
+        for status, results in statuses.items():
+            if status not in success_statuses:
+                for wr in results:
+                    warn_list.append(wr)
+        if len(warn_list) != 0:
+            logger.warning(
+                f"Partial errors:\r\n{json.dumps(q)}\r\n{json.dumps(warn_list)}")
+            result = 2
 
     return result, r, b
 
@@ -83,10 +103,22 @@ def execute_batch(q, blobs, db,
 class ParallelQuery(Parallelizer.Parallelizer):
     """
     **Parallel and Batch Querier for ApertureDB**
+
     This class provides the abstraction for partitioning data into batches,
     so that they may be processed using different threads.
     """
 
+    # 0 is success, 2 is object exists
+    success_statuses = [0, 2]
+
+    @classmethod
+    def setSuccessStatus(cls, statuses: list[int]):
+        cls.success_statuses = statuses
+
+    @classmethod
+    def getSuccessStatus(cls):
+        return cls.success_statuses
+
     def __init__(self, db, dry_run=False):
 
         super().__init__()
@@ -101,13 +133,15 @@ class ParallelQuery(Parallelizer.Parallelizer):
 
         self.commands_per_query = 1
         self.blobs_per_query = 0
+        self.daskManager = None
+        self.batch_command = execute_batch
 
     def generate_batch(self, data):
         """
             Here we flatten the individual queries to run them as
             a single query in a batch
         """
-        q     = [cmd for query in data for cmd in query[0]]
+        q = [cmd for query in data for cmd in query[0]]
         blobs = [blob for query in data for blob in query[1]]
 
         return q, blobs
@@ -122,14 +156,14 @@ class ParallelQuery(Parallelizer.Parallelizer):
         """
         It also provides a way for invoking a user defined function to handle the
         responses of each of the queries executed. This function can be used to process
-        the responses from each of the corresponding queries in :class:`~aperturedb.Parallelizer.Parallelizer`.
+        the responses from each of the corresponding queries in [Parallelizer](/python_sdk/parallel_exec/Parallelizer)
         It will be called once per query, and it needs to have 4 parameters:
         - requests
         - input_blobs
         - responses
         - output_blobs
         Example usage:
-        .. code-block:: python
+        ``` python
             class MyQueries(QueryGenerator):
                 def process_responses(requests, input_blobs, responses, output_blobs):
                     self.requests.extend(requests)
@@ -137,6 +171,7 @@ class ParallelQuery(Parallelizer.Parallelizer):
             loader = ParallelLoader(self.db)
             generator = MyQueries()
             loader.ingest(generator)
+        ```
         """
 
         q, blobs = self.generate_batch(data)
@@ -145,24 +180,42 @@ class ParallelQuery(Parallelizer.Parallelizer):
         worker_stats = {}
         if not self.dry_run:
             response_handler = None
+            strict_response_validation = False
             if hasattr(self.generator, "response_handler") and callable(self.generator.response_handler):
                 response_handler = self.generator.response_handler
-            result, r, b = execute_batch(
-                q, blobs, db, response_handler, self.commands_per_query, self.blobs_per_query)
+            if hasattr(self.generator, "strict_response_validation") and isinstance(self.generator.strict_response_validation, bool):
+                strict_response_validation = self.generator.strict_response_validation
+            result, r, b = self.batch_command(
+                q,
+                blobs,
+                db,
+                ParallelQuery.success_statuses,
+                response_handler,
+                self.commands_per_query,
+                self.blobs_per_query,
+                strict_response_validation=strict_response_validation)
             if result == 0:
                 query_time = db.get_last_query_time()
                 worker_stats["suceeded_commands"] = len(q)
                 worker_stats["suceeded_queries"] = len(data)
+                worker_stats["objects_existed"] = sum(
+                    [v['status'] == 2 for i in r for k, v in i.items()])
             elif result == 1:
                 self.error_counter += 1
                 worker_stats["suceeded_queries"] = 0
                 worker_stats["suceeded_commands"] = 0
+                worker_stats["objects_existed"] = 0
             elif result == 2:
+                # with result 2, some queries might have failed.
+                def filter_per_group(group):
+                    return group.items() if isinstance(group, dict) else {}
                 worker_stats["suceeded_commands"] = sum(
-                    [v['status'] == 0 for i in r for k, v in i.items()])
+                    [v['status'] == 0 for i in r for k, v in filter_per_group(i)])
+                worker_stats["objects_existed"] = sum(
+                    [v['status'] == 2 for i in r for k, v in filter_per_group(i)])
                 sq = 0
                 for i in range(0, len(r), self.commands_per_query):
-                    if all([v['status'] == 0 for j in r[i:i + self.commands_per_query] for k, v in j.items()]):
+                    if all([v['status'] == 0 for j in r[i:i + self.commands_per_query] for k, v in filter_per_group(j)]):
                         sq += 1
                 worker_stats["suceeded_queries"] = sq
         else:
@@ -184,17 +237,31 @@ class ParallelQuery(Parallelizer.Parallelizer):
         for i in range(total_batches):
 
             batch_start = start + i * self.batchsize
-            batch_end   = min(batch_start + self.batchsize, end)
+            batch_end = min(batch_start + self.batchsize, end)
 
             try:
                 self.do_batch(db, generator[batch_start:batch_end])
             except Exception as e:
                 logger.exception(e)
+                logger.warning(
+                    f"Worker {thid} failed to execute batch {i}: [{batch_start},{batch_end}]")
                 self.error_counter += 1
 
             if thid == 0 and self.stats:
                 self.pb.update((i + 1) / total_batches)
 
+    def get_objects_existed(self):
+        return sum([stat["objects_existed"]
+                    for stat in self.actual_stats])
+
+    def get_suceeded_queries(self):
+        return sum([stat["suceeded_queries"]
+                    for stat in self.actual_stats])
+
+    def get_suceeded_commands(self):
+        return sum([stat["suceeded_commands"]
+                    for stat in self.actual_stats])
+
     def query(self, generator, batchsize=1, numthreads=4, stats=False):
         """
         This function takes as input the data to be executed in specified number of threads.
@@ -206,22 +273,35 @@ class ParallelQuery(Parallelizer.Parallelizer):
             stats (bool, optional): Show statistics at end of ingestion. Defaults to False.
         """
 
-        if hasattr(generator, "use_dask") and generator.use_dask:
+        use_dask = hasattr(generator, "use_dask") and generator.use_dask
+        if use_dask:
             self._reset(batchsize=batchsize, numthreads=numthreads)
-            daskmanager = DaskManager(num_workers=numthreads)
+            self.daskmanager = DaskManager(num_workers=numthreads)
 
-            results, self.total_actions_time = daskmanager.run(
-                self.db, generator, batchsize, stats=stats)
+        if hasattr(self, "query_setup"):
+            self.query_setup(generator)
+
+        if use_dask:
+            results, self.total_actions_time = self.daskmanager.run(
+                self.__class__, self.db, generator, batchsize, stats=stats)
+            self.actual_stats = []
             for result in results:
                 if result is not None:
                     self.times_arr.extend(result.times_arr)
                     self.error_counter += result.error_counter
+                    self.actual_stats.append(
+                        {"suceeded_queries": result.suceeded_queries,
+                         "suceeded_commands": result.suceeded_commands,
+                         "objects_existed": result.objects_existed})
             self.total_actions = len(generator.df)
 
             if stats:
                 self.print_stats()
         else:
-            if len(generator) > 0:
+            # allow subclass to do verification
+            if issubclass(type(self), ParallelQuery) and hasattr(self, 'verify_generator') and callable(self.verify_generator):
+                self.verify_generator(generator)
+            elif len(generator) > 0:
                 if isinstance(generator[0], tuple) and isinstance(generator[0][0], list):
                     # if len(generator[0]) > 0:
                     #
@@ -253,8 +333,8 @@ class ParallelQuery(Parallelizer.Parallelizer):
 
         else:
             mean = np.mean(times)
-            std  = np.std(times)
-            tp = 1 / mean * self.numthreads
+            std = np.std(times)
+            tp = 0 if mean == 0 else 1 / mean * self.numthreads
 
             print(f"Avg Query time (s): {mean}")
             print(f"Query time std: {std}")
@@ -270,3 +350,17 @@ class ParallelQuery(Parallelizer.Parallelizer):
                 print(f"Errors (%): {err_perc}")
 
         print("=========================================================")
+
+    def debug_sample(self, **kwargs):
+        """
+        Sample the data to be ingested for debugging purposes.
+        """
+        if "sample_count" in kwargs:
+            sample_count = kwargs["sample_count"]
+        else:
+            sample_count = len(self.generator)
+
+        if sample_count > 0:
+            print("Sample of data to be ingested:")
+            for i in range(sample_count):
+                print(self.generator[i])
