@@ -17,10 +17,9 @@ class PolygonDataCSV(CSVParser.CSVParser):
     This class loads the Polygon Data which is present in a csv file,
     and converts it into a series of aperturedb queries.
 
-    .. note::
-        Is backed by a csv file with the following columns:
-
-            ``IMG_KEY``, [``POLYGON_PROPERTY_1``, ... ``POLYGON_PROPERTY_N``,] [``constraint_POLYGON_PROPERTY_1``, ... ``constraint_POLYGON_PROPERTY_N``,] [``_label``,] ``polygons``
+    :::note Is backed by a csv file with the following columns:
+    ``IMG_KEY``, [``POLYGON_PROPERTY_1``, ... ``POLYGON_PROPERTY_N``,] [``constraint_POLYGON_PROPERTY_1``, ... ``constraint_POLYGON_PROPERTY_N``,] [``_label``,] ``polygons``
+    :::
 
     **IMG_KEY**: identifies the name of the image property that will identify the
     image with which to associate each polygon object. This property should reliably
@@ -33,7 +32,7 @@ class PolygonDataCSV(CSVParser.CSVParser):
 
     **_label**: optionally applies a label to the polygon objects.
 
-    **polygons**: a JSON array of polygon regions. Each polygon region is itself an array of [x,y] vertices that describe the boundary of a single contiguous polygon. See also https://docs.aperturedata.io/parameters/polygons.html.
+    **polygons**: a JSON array of polygon regions. Each polygon region is itself an array of [x,y] vertices that describe the boundary of a single contiguous polygon. See also [Polygon API parameter](query_language/Reference/shared_command_parameters/polygons).
 
     Example csv file::
 
@@ -44,17 +43,18 @@ class PolygonDataCSV(CSVParser.CSVParser):
 
     Example usage:
 
-    .. code-block:: python
+    ``` python
 
         data = PolygonDataCSV("/path/to/PolygonData.csv")
         loader = ParallelLoader(db)
         loader.ingest(data)
+    ```
 
     """
 
-    def __init__(self, filename, df=None):
+    def __init__(self, filename, **kwargs):
 
-        super().__init__(filename, df=df)
+        super().__init__(filename, kwargs=kwargs)
 
         self.props_keys = []
         self.constraints_keys = []
@@ -70,6 +70,13 @@ class PolygonDataCSV(CSVParser.CSVParser):
         self.img_key = self.header[0]
         self.command = "AddPolygon"
 
+    def get_indices(self):
+        return {
+            "entity": {
+                "_Polygon": self.get_indexed_properties()
+            }
+        }
+
     def getitem(self, idx):
         idx = self.df.index.start + idx
 
@@ -77,7 +84,7 @@ class PolygonDataCSV(CSVParser.CSVParser):
 
         img_id = self.df.loc[idx, self.img_key]
 
-        img_ref = (idx % 10000) + 1
+        img_ref = (idx % 99998) + 1
         fi = {
             "FindImage": {
                 "_ref": img_ref,
