@@ -10,7 +10,7 @@ from pydantic import BaseModel
 
 class ObjectType(Enum):
     """
-    Based on : https://docs.aperturedata.io/API-Description.html
+    Based on : [API description](/query_language/Overview/API%20Description)
 
     """
     BLOB = "_Blob"
@@ -221,11 +221,10 @@ class Query():
              blobs: bool = False
              ) -> Query:
         """
-        The specification for a command to be used in a query.
-        https://docs.aperturedata.io/API-Description.html?highlight=query
+        The [specification](/query_language/Overview/API%20Description) for a command to be used in a query.
 
         Args:
-            constraints (Constraints, optional): https://docs.aperturedata.io/parameters/constraints.html . Defaults to None.
+            constraints (Constraints, optional): [Constraints](/query_language/Reference/shared_command_parameters/constraints) . Defaults to None.
             with_class (ObjectType, optional): _description_. Defaults to ObjectType.CUSTOM.
             limit (int, optional): _description_. Defaults to -1.
             sort (Sort, optional): _description_. Defaults to None.
