@@ -1,14 +1,16 @@
 from __future__ import annotations
 from pydantic import BaseModel, Field
 from typing_extensions import Annotated
+from typing import ClassVar
 from uuid import uuid4
 from aperturedb.Query import ObjectType
 
 
 class IdentityModel(BaseModel):
     id: Annotated[str, Field(default_factory=lambda: uuid4().hex)]
-    type: Annotated[ObjectType, Field(
-        description="Object type")] = ObjectType.ENTITY
+    # Change as per the docs for the error
+    # https://docs.pydantic.dev/dev-v2/usage/errors/#model-field-overridden
+    type: ClassVar[ObjectType] = ObjectType.ENTITY
 
 
 class BlobModel(IdentityModel):
