@@ -1,8 +1,16 @@
 import logging
 import json
+import os
+import importlib
+import sys
 
 from aperturedb.Connector import Connector
+from aperturedb.ConnectorRest import ConnectorRest
 from aperturedb import ProgressBar
+from aperturedb.ParallelQuery import execute_batch
+from aperturedb.Configuration import Configuration
+from aperturedb.cli.configure import ls
+
 
 logger = logging.getLogger(__name__)
 
@@ -12,6 +20,62 @@ DESCRIPTOR_CONNECTION_CLASS = "_DescriptorSetToDescriptor"
 DEFAULT_METADATA_BATCH_SIZE = 100_000
 
 
+def import_module_by_path(filepath):
+    """
+    This function imports a module given a path to a python file.
+    """
+    module_name = os.path.basename(filepath)[:-3]
+    spec = importlib.util.spec_from_file_location(module_name, filepath)
+    module = importlib.util.module_from_spec(spec)
+    sys.modules[module_name] = module
+    spec.loader.exec_module(module)
+    return module
+
+
+def __create_connector(configuration: Configuration):
+    if configuration.use_rest:
+        connector = ConnectorRest(
+            host=configuration.host,
+            port=configuration.port,
+            user=configuration.username,
+            password=configuration.password,
+            config=configuration)
+    else:
+        connector = Connector(
+            host=configuration.host,
+            port=configuration.port,
+            user=configuration.username,
+            password=configuration.password,
+            config=configuration)
+    logger.info(f"Connected Using: {configuration}")
+    return connector
+
+
+def create_connector():
+    """
+    **Create a connector to the database.**
+
+    Args:
+        None
+
+    Returns:
+        Connector: The connector to the database.
+    """
+    all_configs = ls(log_to_console=False)
+
+    env_config = os.environ.get("APERTUREDB_CONFIG")
+    ac = all_configs["active"]
+    config = all_configs["local"][ac] if "local" in all_configs else all_configs["global"][ac]
+
+    if env_config is not None:
+        # TODO test me.
+        config = all_configs["global"][env_config] if env_config in all_configs["global"] else all_configs["local"][env_config]
+        return __create_connector(config)
+    # Then check if the local config has active
+    else:
+        return __create_connector(config)
+
+
 class Utils(object):
     """
     **A bunch of helper methods to get information from aperturedb.**
@@ -34,15 +98,24 @@ class Utils(object):
         if self.verbose:
             print(str)
 
+    def execute(self, query, blobs=[], success_statuses=[0]):
+        try:
+            rc, r, b = execute_batch(
+                query, blobs, self.connector, success_statuses=success_statuses)
+        except BaseException as e:
+            logger.error(self.connector.get_last_response_str())
+            raise e
+
+        if rc != 0:
+            raise Exception(f"query failed with result {rc}")
+
+        return r, b
+
     def status(self):
 
         q = [{"GetStatus": {}}]
 
-        try:
-            res, blobs = self.connector.query(q)
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+        self.execute(q)
 
         return self.connector.get_last_response_str()
 
@@ -170,12 +243,8 @@ class Utils(object):
         }]
 
         try:
-            res, blobs = self.connector.query(q)
-            if not self.connector.last_query_ok():
-                logger.error(self.connector.get_last_response_str())
-                return False
+            self.execute(q, success_statuses=[0, 2])
         except:
-            logger.error(self.connector.get_last_response_str())
             return False
 
         return True
@@ -191,12 +260,8 @@ class Utils(object):
         }]
 
         try:
-            res, blobs = self.connector.query(q)
-            if not self.connector.last_query_ok():
-                logger.error(self.connector.get_last_response_str())
-                return False
+            self.execute(q)
         except:
-            logger.error(self.connector.get_last_response_str())
             return False
 
         return True
@@ -241,12 +306,8 @@ class Utils(object):
         if constraints:
             q[0]["FindImage"]["constraints"] = constraints
 
-        try:
-            res, blobs = self.connector.query(q)
-            total_images = res[0]["FindImage"]["count"]
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+        res, _ = self.execute(q)
+        total_images = res[0]["FindImage"]["count"]
 
         return total_images
 
@@ -267,12 +328,8 @@ class Utils(object):
 
         ids = []
 
-        try:
-            res, blobs = self.connector.query(q)
-            total_elements = res[0]["FindEntity"]["batch"]["total_elements"]
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+        res, _ = self.execute(q)
+        total_elements = res[0]["FindEntity"]["batch"]["total_elements"]
 
         batch_size = DEFAULT_METADATA_BATCH_SIZE
         iterations = total_elements // batch_size
@@ -292,13 +349,9 @@ class Utils(object):
 
             q[0]["FindEntity"]["batch"] = batch
 
-            try:
-                res, blobs = self.connector.query(q)
-                ids += [element["_uniqueid"]
-                        for element in res[0]["FindEntity"]["entities"]]
-            except BaseException as e:
-                logger.error(self.connector.get_last_response_str())
-                raise e
+            res, _ = self.execute(q)
+            ids += [element["_uniqueid"]
+                    for element in res[0]["FindEntity"]["entities"]]
 
             if self.verbose:
                 pb.update(i / iterations)
@@ -328,12 +381,8 @@ class Utils(object):
         if constraints:
             q[0]["FindBoundingBox"]["constraints"] = constraints
 
-        try:
-            res, blobs = self.connector.query(q)
-            total_connections = res[0]["FindBoundingBox"]["count"]
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+        res, _ = self.execute(q)
+        total_connections = res[0]["FindBoundingBox"]["count"]
 
         return total_connections
 
@@ -351,23 +400,8 @@ class Utils(object):
         if constraints:
             q[0]["FindEntity"]["constraints"] = constraints
 
-        try:
-            res, blobs = self.connector.query(q)
-            fe = res[0]["FindEntity"]
-
-            if fe["status"] == 1:
-                # TODO: Here we return 0 entities because the query failed.
-                # This is because Find* Command will return status: 1
-                # and no count if no object is found.
-                # We should change the Find* Command to return status: 0
-                # and count: 0 if no object is found.
-                total_entities = 0
-            else:
-                total_entities = fe["count"]
-
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+        res, _ = self.execute(q)
+        total_entities = res[0]["FindEntity"]["count"]
 
         return total_entities
 
@@ -385,12 +419,8 @@ class Utils(object):
         if constraints:
             q[0]["FindConnection"]["constraints"] = constraints
 
-        try:
-            res, blobs = self.connector.query(q)
-            total_connections = res[0]["FindConnection"]["count"]
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+        res, _ = self.execute(q)
+        total_connections = res[0]["FindConnection"]["count"]
 
         return total_connections
 
@@ -405,16 +435,9 @@ class Utils(object):
             }
         }]
 
-        response, arr = self.connector.query(query)
-
-        expected = [{
-            "AddDescriptorSet": {
-                "status": 0,
-            }
-        }]
-
-        if response != expected:
-            logger.error(self.connector.get_last_response_str())
+        try:
+            self.execute(query)
+        except:
             return False
 
         return True
@@ -429,12 +452,8 @@ class Utils(object):
             }
         }]
 
-        try:
-            res, blobs = self.connector.query(q)
-            total_descriptor_sets = res[0]["FindDescriptorSet"]["count"]
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+        res, _ = self.execute(q)
+        total_descriptor_sets = res[0]["FindDescriptorSet"]["count"]
 
         return total_descriptor_sets
 
@@ -450,14 +469,10 @@ class Utils(object):
         }]
 
         sets = []
-        try:
-            res, _ = self.connector.query(q)
-
+        res, _ = self.execute(q)
+        if res[0]["FindDescriptorSet"]["returned"] > 0:
             sets = [ent["_name"]
                     for ent in res[0]["FindDescriptorSet"]["entities"]]
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
 
         return sets
 
@@ -476,12 +491,8 @@ class Utils(object):
         }]
 
         try:
-            res, _ = self.connector.query(q)
-            if not self.connector.last_query_ok():
-                logger.error(self.connector.get_last_response_str())
-                return False
+            self.execute(q)
         except:
-            logger.error(self.connector.get_last_response_str())
             return False
 
         return True
@@ -519,10 +530,9 @@ class Utils(object):
                 }
             }]
 
-            res, _ = self.connector.query(q)
-
-            if not self.connector.last_query_ok():
-                logger.error(self.connector.get_last_response_str())
+            try:
+                self.execute(q)
+            except:
                 return False
 
             self.print(
@@ -549,13 +559,8 @@ class Utils(object):
             }
         }]
 
-        r, _ = self.connector.query(query)
-
-        if not self.connector.last_query_ok():
-            logger.error(self.connector.get_last_response_str())
-            return False
-
         try:
+            r, _ = self.execute(query)
             count = r[0]["DeleteEntity"]["count"]
             self.print(f"Deleted {count} entities")
             self.print(
@@ -580,13 +585,8 @@ class Utils(object):
             }
         }]
 
-        r, _ = self.connector.query(query)
-
-        if not self.connector.last_query_ok():
-            logger.error(self.connector.get_last_response_str())
-            return False
-
         try:
+            r, _ = self.execute(query)
             count = r[0]["DeleteConnection"]["count"]
             self.print(f"Deleted {count} connections")
             self.print(
@@ -670,67 +670,81 @@ class Utils(object):
             }
         }]
 
-        try:
-            res, _ = self.connector.query(q)
-            if not self.connector.last_query_ok():
-                logger.error(self.connector.get_last_response_str())
-            else:
-                total = res[1]["FindDescriptor"]["count"]
-        except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+        res, _ = self.execute(q)
+        total = res[1]["FindDescriptor"]["count"]
 
         return total
 
+    def remove_all_indexes(self):
+        """Remove all indexes from the database.
+
+        This may improve the performance of remove_all_objects.
+        It may improve or degrade the performance of other operations.
+
+        Note that this only removes populated indexes.
+        """
+        def find_indexes(schema):
+            typemap = dict(entities="entity", connections="connection")
+            for typ, data in schema['GetSchema'].items():
+                if typ in typemap and type(data) == dict and 'classes' in data:
+                    for clas, classdata in data['classes'].items():
+                        if 'properties' in classdata and classdata['properties']:
+                            for property_key, (_, indexed, _) in classdata['properties'].items():
+                                if indexed:
+                                    yield {"index_type": typemap[typ], "class": clas, "property_key": property_key}
+
+        try:
+            r, _  = self.execute([{"GetSchema": {}}])
+            schema = r[0]
+            indexes = list(find_indexes(schema))
+            query = [{"RemoveIndex": index} for index in indexes]
+            query.append({"GetSchema": {}})
+            r2, _ = self.execute(query)
+            schema2 = r2[-1]
+            indexes2 = list(find_indexes(schema2))
+            if indexes2:
+                logger.error("Failed to remove all indexes", indexes2)
+                return False
+        except BaseException as e:
+            logger.exception(e)
+            return False
+
+        return True
+
     def remove_all_objects(self):
 
         cmd = {"constraints": {"_uniqueid": ["!=", "0.0.0"]}}
 
-        queries = [
-            [{"DeleteEntity": cmd}],
-            [{"DeleteImage": cmd}],
-            [{"DeleteVideo": cmd}],
-            [{"DeleteBlob": cmd}],
+        transaction = [
+            {"DeleteImage": cmd},
+            {"DeleteVideo": cmd},
+            {"DeleteBlob": cmd},
+            {"DeleteClip": cmd},
             # DeleteDescriptorSet also deletes the descriptors
-            [{"DeleteDescriptorSet": cmd}],
+            {"DeleteDescriptorSet": cmd},
 
             # All the following should be deleted automatically once the
             # related objects are deleted.
             # We keep them here until ApertureDB fully implements this.
-            [{"DeleteBoundingBox": cmd}],
-            [{"DeletePolygon": cmd}],
-            [{"DeleteFrame": cmd}],
+            {"DeleteBoundingBox": cmd},
+            {"DeletePolygon": cmd},
+            {"DeleteFrame": cmd},
+            {"DeleteEntity": cmd},
+            {"GetSchema": {"refresh": True}}
         ]
 
         try:
-            for q in queries:
-                response, _ = self.connector.query(q)
-                if not self.connector.last_query_ok():
-                    logger.error(self.connector.get_last_response_str())
-                    return False
-
-            response, _ = self.connector.query(
-                [{"GetSchema": {"refresh": True}}])
-
-            if not self.connector.last_query_ok():
-                logger.error(self.connector.get_last_response_str())
-                return False
-
-            entities    = response[0]["GetSchema"]["entities"]
-            connections = response[0]["GetSchema"]["connections"]
-
-            if entities is not None:
-                logger.error("Entities not removed completely")
-                logger.error(self.connector.get_last_response_str())
-                return False
-
-            if connections is not None:
-                logger.error("Connections not removed completely")
-                logger.error(self.connector.get_last_response_str())
-                return False
-
+            response, _ = self.execute(transaction)
+            schema = response[-1]["GetSchema"]
+            if schema["status"] != 0:
+                logger.error("status is non-zero", str(response))
+            elif schema["connections"] is not None:
+                logger.error("connections is not None", str(response))
+            elif schema["entities"] is not None:
+                logger.error("entities is not None", str(response))
+            else:
+                return True
         except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
-            raise e
+            logger.exception(e)
 
-        return True
+        return False
