@@ -1,22 +1,34 @@
+import pytest
+import mark_checker
+
 from collections import namedtuple
 import os
 from typing import List, Tuple
 import zipfile
 import pandas as pd
-import os
-from kaggle.api.kaggle_api_extended import KaggleApi
-from aperturedb.KaggleData import KaggleData
 from unittest.mock import MagicMock, patch
 
+pytestmark = [pytest.mark.kaggle, pytest.mark.remote_credentials]
+
+if mark_checker.is_enabled(("kaggle", "remote_credentials")):
+    global KaggleApi
+    global KaggleData
+    from kaggle.api.kaggle_api_extended import KaggleApi
+    from aperturedb.KaggleData import KaggleData
+else:
+    # mock class so it will load, we aren't using it.
+    class KaggleData():
+        pass
 kf = namedtuple("files", ["files"])
 file = namedtuple("file", ["fileType", "totalBytes", "name"])
 
 
 def custom_download(workdir, files):
-    def dload_zip(dataset, path, unzip):
+    def dload_zip(dataset, path, quiet, unzip):
         assert(dataset in workdir)
         assert(path == workdir)
         assert(not unzip)
+        assert(not quiet)
         # Simulate a download from kaggle.
         # the kaggle API leads to a zipped dataset downloaded into the dataset_ref path.
         os.makedirs(workdir, exist_ok=True)
